import socket
import subprocess
import platform


def check_port_in_use(port):
    system = platform.system()
    print(f"尝试使用socket方式检查端口:{port}")
    check_dict = {}
    is_port_in_use = False

    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.bind(('localhost', port))
        sock.close()
        check_dict["Socket"] = False
    except socket.error:
        # print(f"socket检查无法使用端口号{port}，可能被占用")
        check_dict["Socket"] = True
        is_port_in_use = True  # 如果出现socket错误，标记端口被占用

    print(f"尝试使用命令行方式检查端口:{port}")

    # 对于Unix-like系统（包括macOS和Linux），尝试使用lsof命令
    if system in ["Linux", "Darwin"]:
        print("system in [Linux, Darwin]")
        command = f'lsof -i :{port}'
        output = subprocess.check_output(command, shell=True, text=True)
        result = bool(output.strip())
        check_dict["Command_Line_lsof"] = result
        is_port_in_use |= result  # 如果lsof检查结果显示端口被占用，更新标志位

    # 对于Windows系统
    elif system == "Windows":
        print("system == Windows")
        command = f"netstat -ano | findstr :{port}"
        output = subprocess.run(command, shell=True, capture_output=True, text=True)
        result = bool(output.stdout.strip())
        check_dict["Command_Line_netstat"] = result
        is_port_in_use |= result  # 如果netstat检查结果显示端口被占用，更新标志位
    else:
        print("system == None")
        raise ValueError(f"Unsupported operating system: {system}")

    # 返回端口占用情况字典及是否被占用的布尔值
    return check_dict, is_port_in_use


# 示例
port_to_check = 1883
check_info, is_port_used = check_port_in_use(port_to_check)
print(check_info)
print("端口是否使用:", is_port_used)


import asyncio
import os
import pathlib
import sys
import threading
import time

from easysmart.manager import Manager
import zeroconf
import paho

from easysmart.mdns.mdns import mdns_register
from easysmart.mdns.mdns_async import mdns_async_register
from easysmart.mqtt_server.mqtt_server import start_emqx_server
from easysmart.web.webmain import WebServer


async def test_publish(manager):
    while True:
        await asyncio.sleep(5)
        print('publish')
        await manager.publish('/test/', 'hello world', 0)


def start_server(root_path=None, block=True, services=None):
    if sys.platform.lower() == "win32" or os.name.lower() == "nt":
        from asyncio import set_event_loop_policy, WindowsSelectorEventLoopPolicy
        set_event_loop_policy(WindowsSelectorEventLoopPolicy())
    if root_path is None:
        # get the root path of this project
        root_path = pathlib.Path(__file__).parent.parent.absolute()
        root_path = pathlib.Path(os.path.expanduser('~') + '/easysmart/')
    # 如果不存在则创建
    if not root_path.exists():
        root_path.mkdir()
    print(f'root path is {root_path}')
    # # start the manager
    # asyncio.gather(start_emqx_server(root_path))

    # create thread to register mdns
    thread = threading.Thread(target=mdns_register)
    thread.daemon = True
    thread.start()



    try:
        loop = asyncio.get_event_loop()
    except:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    # loop.run_until_complete(mdns_async_register())
    # manager = Manager()
    # asyncio.gather(manager.async_loop_start())
    # web_manager = WebServer(manager)
    # asyncio.gather(web_manager.web_start())
    # # asyncio.gather(test_publish(manager))

    main_manager = Manager()

    loop.create_task(async_start_server(root_path, main_manager, services=services), )
    print('main manager start')
    if block:
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            print("stop server")
    else:
        return main_manager, loop
    # asyncio.run(main())


async def async_start_server(root_path=None, main_manager=None, services: list = None):
    print(f'async_start_server')
    print(f'root path is {root_path}')
    # asyncio.create_task(mdns_async_register())
    asyncio.create_task(start_emqx_server(root_path))
    asyncio.create_task(main_manager.async_loop_start())
    if services is None:
        services = []
    for service in services:
        s = service(main_manager)
        asyncio.create_task(s.start())
    while True:
        await asyncio.sleep(1)



import asyncio

from easysmart import Manager


class AutomaticService(object):
    def __init__(self, manager: 'Manager'):
        self.manager = manager

    async def run(self):
        pass

    async def start(self):
        ...

    async def on_device_connect(self):
        pass

    async def on_device_value_change(self):
        pass

    async def on_device_action(self):
        pass

    async def on_device_disconnect(self):
        pass

    async def on_message(self, topic, payload):
        pass


'''
1. 触发条件
时间，设备动作，设备属性变化，设备连接，设备断开
2. 执行动作
'''
